// Module: c_node.v
// Description: c_node or child node for the FPGA HyperSplit tree.
// Author: Jeffrey Fong
// Date: 2010.05.05

module c_node #(parameter DATA_WIDTH = 64, P_Length = 28, Level = 2, mem = Level)
(clk, pkt_in, node_data, found_in, pkt_nxt, nxt_node, found_out, we, addr_a, din_a, dout_a, stage);
input clk, found_in, we;
input [5:0] stage;
input [P_Length-1:0] addr_a;
input [63:0] node_data, din_a;
input [103:0] pkt_in;
output found_out;
output [63:0] nxt_node, dout_a;
output [103:0] pkt_nxt;

wire [P_Length-1:0] addr_to_ram;
wire [DATA_WIDTH-1:0] ram_in;
i_node #(.DATA_WIDTH(DATA_WIDTH), .P_Length(P_Length))
		node(clk, pkt_in, node_data, found_in, pkt_nxt, nxt_node, found_out, addr_to_ram, ram_in);

reg m_we;
always@(stage or we) begin
	if((we == 1'b1) &(stage == Level))
		m_we = 1'b1;
	else
		m_we = 1'b0;
end

ram #(.ADDR_WIDTH((mem))) ram1(clk, m_we, addr_a, addr_to_ram, din_a, dout_a, ram_in);

endmodule
