// Module: full_path.v
// Description: Top-level of the hardware/FPGA implementation of HyperSplit
// Author: Jeffrey Fong
// Date: 2010.05.05

module full_path(clk, pkt_in, pkt_out, nxt_node, we, addr_a, din_a, stage);
input clk, we;
input [5:0] stage;
input [27:0] addr_a;
input [63:0] din_a;
input [103:0] pkt_in;
output [63:0] nxt_node;
output [103:0] pkt_out;
reg [103:0] pkt_out;

wire [103:0] pkt_nxt1, pkt_nxt2, pkt_nxt3, pkt_nxt4, pkt_nxt5, pkt_nxt6, pkt_nxt7;
wire [103:0] pkt_nxt8, pkt_nxt9, pkt_nxt10, pkt_nxt11, pkt_nxt12, pkt_nxt13;
wire [103:0] pkt_nxt14, pkt_nxt15, pkt_nxt16, pkt_nxt17, pkt_nxt18, pkt_nxt19;
wire [103:0] pkt_nxt20, pkt_nxt21, pkt_nxt22, pkt_nxt23, pkt_nxt24, pkt_nxt25;
wire [63:0] nxt_node1, nxt_node2, nxt_node3, nxt_node4, nxt_node5, nxt_node6, nxt_node7;
wire [63:0] nxt_node8, nxt_node9, nxt_node10, nxt_node11, nxt_node12, nxt_node13;
wire [63:0] nxt_node14, nxt_node15, nxt_node16, nxt_node17, nxt_node18, nxt_node19;
wire [63:0] nxt_node20, nxt_node21, nxt_node22, nxt_node23, nxt_node24, nxt_node25;

r_node rnode1(clk, pkt_in, pkt_nxt1, nxt_node1, found_out1, we, addr_a, din_a, dout_a1, stage);

c_node #(.Level(2)) cnode2(.clk(clk), .pkt_in(pkt_nxt1), .node_data(nxt_node1), .found_in(found_out1), 
			  .pkt_nxt(pkt_nxt2), .nxt_node(nxt_node2), .found_out(found_out2),
			  .we(we), .addr_a(addr_a), .din_a(din_a), .dout_a(dout_a2), .stage(stage));

c_node #(.Level(3)) cnode3(.clk(clk), .pkt_in(pkt_nxt2), .node_data(nxt_node2), .found_in(found_out2), 
			  .pkt_nxt(pkt_nxt3), .nxt_node(nxt_node3), .found_out(found_out3),
			  .we(we), .addr_a(addr_a), .din_a(din_a), .dout_a(dout_a3), .stage(stage));

c_node #(.Level(4)) cnode4(.clk(clk), .pkt_in(pkt_nxt3), .node_data(nxt_node3), .found_in(found_out3), 
			  .pkt_nxt(pkt_nxt4), .nxt_node(nxt_node4), .found_out(found_out4),
			  .we(we), .addr_a(addr_a), .din_a(din_a), .dout_a(dout_a4), .stage(stage));

c_node #(.Level(5)) cnode5(.clk(clk), .pkt_in(pkt_nxt4), .node_data(nxt_node4), .found_in(found_out4), 
			  .pkt_nxt(pkt_nxt5),.nxt_node(nxt_node5), .found_out(found_out5),
			  .we(we), .addr_a(addr_a), .din_a(din_a), .dout_a(dout_a5), .stage(stage));

c_node #(.Level(6)) cnode6(.clk(clk), .pkt_in(pkt_nxt5), .node_data(nxt_node5), .found_in(found_out5), 
			  .pkt_nxt(pkt_nxt6),.nxt_node(nxt_node6), .found_out(found_out6),
			  .we(we), .addr_a(addr_a), .din_a(din_a), .dout_a(dout_a6), .stage(stage));

c_node #(.Level(7)) cnode7(.clk(clk), .pkt_in(pkt_nxt6), .node_data(nxt_node6), .found_in(found_out6), 
			  .pkt_nxt(pkt_nxt7),.nxt_node(nxt_node7), .found_out(found_out7),
			  .we(we), .addr_a(addr_a), .din_a(din_a), .dout_a(dout_a7), .stage(stage));

c_node #(.Level(8)) cnode8(.clk(clk), .pkt_in(pkt_nxt7), .node_data(nxt_node7), .found_in(found_out7), 
			  .pkt_nxt(pkt_nxt8),.nxt_node(nxt_node8), .found_out(found_out8),
			  .we(we), .addr_a(addr_a), .din_a(din_a), .dout_a(dout_a8), .stage(stage));

c_node #(.Level(9)) cnode9(.clk(clk), .pkt_in(pkt_nxt8), .node_data(nxt_node8), .found_in(found_out8), 
			  .pkt_nxt(pkt_nxt9),.nxt_node(nxt_node9), .found_out(found_out9),
			  .we(we), .addr_a(addr_a), .din_a(din_a), .dout_a(dout_a9), .stage(stage));

c_node #(.Level(10)) cnode10(.clk(clk), .pkt_in(pkt_nxt9), .node_data(nxt_node9), .found_in(found_out9), 
			  .pkt_nxt(pkt_nxt10),.nxt_node(nxt_node10), .found_out(found_out10),
			  .we(we), .addr_a(addr_a), .din_a(din_a), .dout_a(dout_a10), .stage(stage));

c_node #(.Level(11)) cnode11(.clk(clk), .pkt_in(pkt_nxt10), .node_data(nxt_node10), .found_in(found_out10), 
			  .pkt_nxt(pkt_nxt11),.nxt_node(nxt_node11), .found_out(found_out11),
			  .we(we), .addr_a(addr_a), .din_a(din_a), .dout_a(dout_a11), .stage(stage));

c_node #(.Level(12)) cnode12(.clk(clk), .pkt_in(pkt_nxt11), .node_data(nxt_node11), .found_in(found_out11), 
			  .pkt_nxt(pkt_nxt12),.nxt_node(nxt_node12), .found_out(found_out12),
			  .we(we), .addr_a(addr_a), .din_a(din_a), .dout_a(dout_a12), .stage(stage));

c_node #(.Level(13), .mem(12)) cnode13(.clk(clk), .pkt_in(pkt_nxt12), .node_data(nxt_node12), .found_in(found_out12), 
			  .pkt_nxt(pkt_nxt13),.nxt_node(nxt_node13), .found_out(found_out13),
			  .we(we), .addr_a(addr_a), .din_a(din_a), .dout_a(dout_a13), .stage(stage));

c_node #(.Level(14), .mem(12)) cnode14(.clk(clk), .pkt_in(pkt_nxt13), .node_data(nxt_node13), .found_in(found_out13), 
			  .pkt_nxt(pkt_nxt14),.nxt_node(nxt_node14), .found_out(found_out14),
			  .we(we), .addr_a(addr_a), .din_a(din_a), .dout_a(dout_a14), .stage(stage));

c_node #(.Level(15), .mem(12)) cnode15(.clk(clk), .pkt_in(pkt_nxt14), .node_data(nxt_node14), .found_in(found_out14), 
			  .pkt_nxt(pkt_nxt15),.nxt_node(nxt_node15), .found_out(found_out15),
			  .we(we), .addr_a(addr_a), .din_a(din_a), .dout_a(dout_a15), .stage(stage));

c_node #(.Level(16), .mem(11)) cnode16(.clk(clk), .pkt_in(pkt_nxt15), .node_data(nxt_node15), .found_in(found_out15), 
			  .pkt_nxt(pkt_nxt16),.nxt_node(nxt_node16), .found_out(found_out16),
			  .we(we), .addr_a(addr_a), .din_a(din_a), .dout_a(dout_a16), .stage(stage));

c_node #(.Level(17), .mem(10)) cnode17(.clk(clk), .pkt_in(pkt_nxt16), .node_data(nxt_node16), .found_in(found_out16), 
			  .pkt_nxt(pkt_nxt17),.nxt_node(nxt_node17), .found_out(found_out17),
			  .we(we), .addr_a(addr_a), .din_a(din_a), .dout_a(dout_a17), .stage(stage));

c_node #(.Level(18), .mem(8)) cnode18(.clk(clk), .pkt_in(pkt_nxt17), .node_data(nxt_node17), .found_in(found_out17), 
			  .pkt_nxt(pkt_nxt18),.nxt_node(nxt_node18), .found_out(found_out18),
			  .we(we), .addr_a(addr_a), .din_a(din_a), .dout_a(dout_a18), .stage(stage));

c_node #(.Level(19), .mem(5)) cnode19(.clk(clk), .pkt_in(pkt_nxt18), .node_data(nxt_node18), .found_in(found_out18), 
			  .pkt_nxt(pkt_nxt19),.nxt_node(nxt_node19), .found_out(found_out19),
			  .we(we), .addr_a(addr_a), .din_a(din_a), .dout_a(dout_a19), .stage(stage));
			  
c_node #(.Level(20), .mem(1)) cnode20(.clk(clk), .pkt_in(pkt_nxt19), .node_data(nxt_node19), .found_in(found_out19), 
			  .pkt_nxt(pkt_nxt20),.nxt_node(nxt_node20), .found_out(found_out20),
			  .we(we), .addr_a(addr_a), .din_a(din_a), .dout_a(dout_a20), .stage(stage));


always@(posedge clk) begin
	pkt_out <= pkt_nxt20;
end

assign nxt_node = nxt_node20;
			  
endmodule
