// Module: i_node.v
// Description: HyperSplit i_node or independent node module.  This implements
//				the fundamental comparision and ram lookup logic.
// Author: Jeffrey Fong
// Date: 2010.05.05

module i_node #(parameter DATA_WIDTH = 64, P_Length = 28)
(clk, pkt_in, node_data, found_in, pkt_nxt, nxt_node, found_out, addr_to_ram, ram_in);
input clk, found_in;
input [63:0] node_data, ram_in;
input [103:0] pkt_in;
output found_out;
output [P_Length-1:0] addr_to_ram;
output[103:0] pkt_nxt;
output[63:0] nxt_node;

reg found_out, found;
reg [31:0] rule;
reg[27:0] addr_to_ram;  // addr_a
reg [103:0] pkt_reg;
//alias
wire [31:0] s_addr, d_addr;
wire [15:0] s_port, d_port;
wire [7:0] protocol;
assign s_addr = pkt_reg[103:72];
assign d_addr = pkt_reg[71:40];
assign s_port = pkt_reg[39:24];
assign d_port = pkt_reg[23:8];
assign protocol = pkt_reg[7:0];

//alias
wire [P_Length-1:0] node_addr;
wire [3:0] field_select;
wire [31:0] field_val;
assign node_addr = node_data[DATA_WIDTH-1:DATA_WIDTH-P_Length];
assign field_select = node_data[DATA_WIDTH-P_Length-1: DATA_WIDTH-P_Length-4];
assign field_val = node_data[DATA_WIDTH-P_Length-4-1:0];

always@(posedge clk) begin
		pkt_reg <= pkt_in;
end

reg [31:0] sel_field;
always@(field_select or pkt_reg or s_port or d_port or protocol or s_addr or d_addr) begin
	case(field_select)
		4'b0001: sel_field = s_addr;
		4'b0010: sel_field = d_addr;
		4'b0011: sel_field = {16'b0, s_port};
		4'b0100: sel_field = {16'b0, d_port};
		4'b0101: sel_field = {24'b0, protocol};
		default: sel_field = 32'b0;
	endcase
end

always@(field_val or sel_field or node_addr) begin
	if(field_val >= sel_field)
		addr_to_ram = node_addr;
	else
		addr_to_ram = node_addr+1;
end

always@(field_select or found_in) begin
	if(field_select == 4'b0000)
		found = 1'b1;
	else if(found_in == 1'b1)
		found = 1'b1;
	else
		found = 1'b0;
end

always@(posedge clk) begin
	rule <= field_val;
end

always@(posedge clk) begin
	found_out <= found;
end

assign pkt_nxt = pkt_reg;
assign nxt_node = (found_out == 1'b0) ? ram_in : {ram_in[63:32],rule};

endmodule
