// Module: r_node.v
// Description: HyperSplit Root Node which contains the first 3 nodes of the tree.
// 				Instantiates i_node used for making decision.
// Author: Jeffrey Fong
// Date: 2010.05.05

module r_node #(parameter DATA_WIDTH = 64, P_Length = 28, Level =1, mem = 1)
(clk, pkt_in, pkt_nxt, nxt_node, found_out, we, addr_a, din_a, dout_a, stage);
input clk, we;
input [5:0] stage;
input [P_Length-1:0] addr_a;
input [63:0] din_a;
input [103:0] pkt_in;
output found_out;
output [63:0] nxt_node, dout_a;
output [103:0] pkt_nxt;

reg[DATA_WIDTH-1:0] root_node;
always@(posedge clk) begin
	if(we == 1'b1)
		if((addr_a == 0) & (stage == 0))
			root_node <= din_a;
end

reg m_we;
always@(we or stage) begin
	if((we == 1'b1) & (stage == 6'd1))
		m_we = 1'b1;
	else
		m_we = 1'b0;
end

wire [P_Length-1:0] addr_to_ram;
wire [DATA_WIDTH-1:0] ram_in;
i_node #(.DATA_WIDTH(DATA_WIDTH), .P_Length(P_Length))
		node (clk, pkt_in, root_node, 1'b0, pkt_nxt, nxt_node, found_out, addr_to_ram, ram_in);

ram #(.ADDR_WIDTH(mem)) ram1(clk, m_we, addr_a, addr_to_ram, din_a, dout_a, ram_in);

endmodule
