// Module: ram.v
// Description: Basic Dual-port ram module
// Author: Jeffrey Fong
// Date: 2010.05.05

module ram #(parameter ADDR_WIDTH = 8, DATA_WIDTH = 64) (clk, we, addr_a, addr_b, din_a, dout_a, dout_b);
input clk, we;
input [ADDR_WIDTH-1:0] addr_a, addr_b;
input [DATA_WIDTH-1:0] din_a;
output[DATA_WIDTH-1:0] dout_a, dout_b;

reg [DATA_WIDTH-1:0] int_ram [2**ADDR_WIDTH-1:0];
reg [ADDR_WIDTH-1:0] addr_a_reg, addr_b_reg;

always@(posedge clk) begin
	if(we == 1'b1)
		int_ram[addr_a] <= din_a;
	addr_a_reg <= addr_a;
	addr_b_reg <= addr_b;
end

assign dout_a = int_ram[addr_a_reg];
assign dout_b = int_ram[addr_b_reg];

endmodule
