// Module: testbench.v
// Description: Test bench for fullpath of HyperSplit
// Author: Jeffrey Fong
// Date: 2010.05.05

`timescale 10ns/1ns
module testbench();

reg clk;
initial begin
	clk <= 1'b0;
end
always begin
	#1 clk <= ~clk;
end

reg we;
reg [5:0] stage;
reg [27:0] addr_a;
reg [63:0] din_a;
reg [103:0] pkt_in;
wire [63:0] nxt_node;
wire [103:0] pkt_out;

full_path fp1(clk, pkt_in, pkt_out, nxt_node, we, addr_a, din_a, stage);

//alias signals for loading testbench file
// loading signals
reg [27:0] pointer;
reg [3:0] field;
reg [31:0] field_val;
// packet signals
reg [31:0] s_addr, d_addr;
reg [15:0] s_port, d_port;
reg [7:0] protocol;
integer k, fp, cnt, temp;
initial begin
	pkt_in <= 104'b0;
	addr_a <= 28'b0;
	din_a <= 64'b0;
	we = 1'b0;
	stage = 6'b0;
	// load data into ram
	#200;
	
	initialize_rules();
	
	#50;
	$monitor("Time: %d, Packet Out: %d, Rule: %d", $time, pkt_out, nxt_node);
	test_packets();
end

initial begin
	#100000 $stop;
end


// ============================================================
//  Task definitions
// ============================================================

task test_packets();
begin
	fp = $fopen("rules/packet_test.txt", "r");
	$fscanf(fp, "%d", cnt);
	for( k=0; k < cnt; k=k+1) begin
		protocol = 8'b0;
		$fscanf(fp, "%d", temp);
		$fscanf(fp, "%d", s_addr);
		$fscanf(fp, "%d", d_addr);
		$fscanf(fp, "%d", s_port);
		$fscanf(fp, "%d", d_port);
		//$display("Testing packet! s_addr: %d, d_addr: %d, s_port: %d, d_port: %d", s_addr, d_addr, s_port, d_port);
		pkt_in <= {s_addr,d_addr,s_port,d_port,protocol};
		#2;
	end
	$fclose(fp);
end
endtask
	

task initialize_rules();
begin
	fp = $fopen("rules/0.txt", "r");
	load_ram(fp, 0);
	fp = $fopen("rules/1.txt", "r");
	load_ram(fp, 1);
	fp = $fopen("rules/2.txt", "r");
	load_ram(fp, 2);
	fp = $fopen("rules/3.txt", "r");
	load_ram(fp, 3);
	fp = $fopen("rules/4.txt", "r");
	load_ram(fp, 4);
	fp = $fopen("rules/5.txt", "r");
	load_ram(fp, 5);
	fp = $fopen("rules/6.txt", "r");
	load_ram(fp, 6);
	fp = $fopen("rules/7.txt", "r");
	load_ram(fp, 7);
	fp = $fopen("rules/8.txt", "r");
	load_ram(fp, 8);
	fp = $fopen("rules/9.txt", "r");
	load_ram(fp, 9);
	fp = $fopen("rules/10.txt", "r");
	load_ram(fp, 10);
	fp = $fopen("rules/11.txt", "r");
	load_ram(fp, 11);
	fp = $fopen("rules/12.txt", "r");
	load_ram(fp, 12);
	fp = $fopen("rules/13.txt", "r");
	load_ram(fp, 13);
end
endtask
	

task load_ram(input integer file, nstage);
begin
	$fscanf(file, "%d", cnt);
	for( k=0; k < cnt; k=k+1) begin
		stage = nstage;
		we = 1'b1;
		$fscanf(file, "%d", temp);
		$fscanf(file, "%d", pointer);
		$fscanf(file, "%d", field);
		$fscanf(file, "%d", field_val);
		$display("Stage: %d, Line: %d, Pointer: %d, Field: %d, Field_value: %d", nstage, temp, pointer, field, field_val);
		din_a = {pointer, field, field_val};
		addr_a = k;
		#2;
	end
	$fclose(file);
	we = 1'b0;
end
endtask

endmodule
